// This file is part of the PiDuce project (http://www.cs.unibo.it/Piduce/)
// developed at the Department of Computer Science of Bologna.
// Authors: Samuele Carpineti, Cosimo Laneve, Luca Padovani
//
// See Copyright for the status of this software.

using System;
using PiDuce.Common;

namespace PiDuce.Machine
{
#if false
	public class MachineSchemaToTypeVisitor : SchemaToTypeVisitor
	{
		protected override void IllegalSchema(ISchema s, string msg)
		{ throw new ApplicationException("illegal schema: " + msg); }
	}
#endif

  public class SchemaFromLabelVisitor : ISchemaVisitor
  {
    private readonly string label;
    private LabelledSchema schema;
    public LabelledSchema Schema { get { return schema; } }

    public SchemaFromLabelVisitor(string label)
    {
      this.label = label;
    }
    public void VisitLabelledSchema(LabelledSchema s)
    {
      if (s.Labels.Includes(LabelSet.Singleton(label)))
        this.schema = s;
    }
    public void VisitSequenceSchema(SequenceSchema s)
    {
      s.Head.Accept(this);
      if (Schema == null)
        s.Tail.Accept(this);
    }

    public void VisitUnionSchema(UnionSchema s)
    {
      s.Left.Accept(this);
      if (Schema == null)
        s.Right.Accept(this);
    }

    public void VisitStarSchema(StarSchema s)
    { s.Content.Accept(this); }

    public void VisitPlusSchema(PlusSchema s)
    { s.Content.Accept(this); }

    public void VisitRepetitionSchema(RepetitionSchema s)
    { s.Content.Accept(this); }

    public void VisitVoidSchema(VoidSchema s)
    { }

    public void VisitBasicSchema(BasicSchema s)
    { }

    public void VisitBasicSchemaLiteralSchema(BasicSchemaLiteralSchema s)
    { }

    public void VisitConstantSchema(ConstantSchema s)
    { s.Entry.Schema.Accept(this); }

    public void VisitChannelSchema(ChannelSchema s)
    { }

    public void VisitFunctionSchema(FunctionSchema s)
    { }

    public void VisitErrorSchema(ErrorSchema s)
    { }

    public void VisitBindSchema(BindSchema s)
    { s.Content.Accept(this); }

    public void VisitRecordSchema(RecordSchema s)
    { }
  }
}