// This file is part of the PiDuce project (http://www.cs.unibo.it/Piduce/)
// developed at the Department of Computer Science of Bologna.
// Authors: Samuele Carpineti, Cosimo Laneve, Luca Padovani
//
// See Copyright for the status of this software.

using System;
using System.Xml;
using System.Collections.Generic;
using System.Text;
using System.Collections;
using System.IO;
using PiDuce.Common;
using PiDuce.Web;
using PiDuce.Types;
using System.Diagnostics;

namespace PiDuce.Machine
{
  public class SoapHandler
  {
    private string targetNamespace;
    private ISchema schema;
    private ValueToXmlVisitor visitor;
    private ValueUtil valueUtil;

    public ISchema Schema { get { return schema; } }
    public ValueToXmlVisitor Visitor { get { return visitor; } }
    public string TargetNamespace { get { return targetNamespace; } }
    public ValueUtil ValueUtil { get { return valueUtil; } }
   
    public SoapHandler(string targetNamespace, ISchema schema, ValueToXmlVisitor visitor, ValueUtil valueUtil)
    {
      this.targetNamespace = targetNamespace;
      this.schema = schema;
      this.visitor = visitor;
      this.valueUtil = valueUtil;
    }

    public SoapHandler(ValueUtil valueUtil, ISchema schema)
    {
      this.valueUtil = valueUtil;
      this.schema = schema;
    }

    public string AddSoapToMessage(IValue v)
    {
      XmlWriter xml = visitor.Xml;
      xml.WriteStartElement("soap:Envelope");
      xml.WriteAttributeString("xmlns:" + WebNamespace.XmlSchemaInstancePrefix, WebNamespace.XmlSchemaInstance);
      xml.WriteAttributeString("xmlns:" + WebNamespace.XmlSchemaNamespacePrefix, WebNamespace.XmlSchemaNamespace);
      xml.WriteAttributeString("xmlns:" + WebNamespace.SoapEncodingStylePrefix, WebNamespace.SoapEncNamespace);
      xml.WriteAttributeString("xmlns:tns", this.TargetNamespace);
      xml.WriteAttributeString("xmlns:" + WebNamespace.SoapNamespacePrefix, WebNamespace.SoapNamespace + "envelope/");
      xml.WriteStartElement("soap:Body");
      //value belongs to current machine namespace
      v.Accept(this.Visitor);
      xml.WriteEndElement();
      xml.WriteEndElement();
      return visitor.Text;
    }

    public static XmlNodeList RemoveSoap(String soap)
    {
      XmlDocument xml = new XmlDocument();
      xml.LoadXml(soap);
      XmlNode body = xml.GetElementsByTagName("Body", "http://schemas.xmlsoap.org/soap/envelope/")[0];
      //Console.WriteLine(soap);
      return body.ChildNodes;
    }

    public IValue ReadResponse(XmlNodeList xml)
    { 
      IValue value = this.ValueUtil.XmlToIValue(xml, Schema);
      return value;
    }
    
  }
}
