// This file is part of the PiDuce project (http://www.cs.unibo.it/Piduce/)
// developed at the Department of Computer Science of Bologna.
// Authors: Samuele Carpineti, Cosimo Laneve, Luca Padovani
//
// See Copyright for the status of this software.

using System;
using System.Collections.Generic;
using System.Collections;
using PiDuce.Common;
using System.Diagnostics;

namespace PiDuce.Types
{
	public class DeterminednessChecker
	{
		private readonly HandlesOfSchemaVisitor handlesOf = new HandlesOfSchemaVisitor(new EmptynessChecker());

		private IList<IHandle> GetHandles(ISchema s)
		{ return handlesOf.GetHandles(s); }

		public bool IsDetermined(ISchema s)
		{
			IList<IHandle> handles = GetHandles(s);
			ILabelSet labels = LabelSet.Empty();
			foreach (IHandle handle in handles)
				if (handle is Handle<LabelledSchema>) {
					Handle<LabelledSchema> lh = (Handle<LabelledSchema>) handle;
					if (lh.Head.Labels.Intersects(labels) || !IsDetermined(lh.Head.Content) || !IsDetermined(lh.Tail))
						return false;
					labels = LabelSet.Union(lh.Head.Labels, labels);
				} else if (handle is NonVoidHandle) {
					if (!IsDetermined(((NonVoidHandle) handle).Tail))
						return false;
				}
			return true;
		}	
	}
}
